<?php

use Delo\DeloDebugInfo;
use Delo\DeloSettings;

/**
 * Class ModelExtensionPaymentDelo
 */
class ModelExtensionPaymentDelo extends Model
{
    /** @var DeloSettings */
    private $settingsModel;
    /** @var DeloDebugInfo */
    private $debugInfo;

    /**
     * ModelExtensionPaymentDelo constructor.
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->library('Delo/DeloErrors');
        $this->load->library('Delo/DeloLog');
        $this->load->library('Delo/DeloRequirements');
        $this->load->library('Delo/DeloSettings');
        $this->load->library('Delo/DeloDebugInfo');
        $this->load->language('extension/payment/delo');
        $this->settingsModel = $this->registry->get('DeloSettings');
        $this->debugInfo = DeloDebugInfo::getInstance($registry);
    }

    /**
     * Этот метод вызывается из стандартного opencart контроллера корзины - место выбора способа оплаты
     * Возвращает массив с именем способа оплаты, его уникальный код, условия, сортировку
     * @param string $address адрес покупателя, заполненый в корзине
     * @param float $total сумма заказа
     * @return array
     * @throws Exception
     */
    public function getMethod($address, $total)
    {
        $settings = $this->settingsModel->getSettingsValue();
        $data = [
            'code'           => 'delo',
            'title'          => $settings['title'],
            'terms'          => $settings['terms'],
            'sort_order'     => 0, // Место среди способов оплаты - сортировка, 0 - первый
        ];
        return $data;
    }

    /**
     * Достает из БД информацию о заказе и возвращает ее
     * @param int $orderId
     * @return array|null данные по заказу
     */
    public function getOrder($orderId)
    {
        $this->load->model('checkout/order');
        $order = $this->model_checkout_order->getOrder($orderId);
        return $order;
    }

    /**
     * Возвращает кусок xml с <result>
     * @param int $code 1 – Успех. Продолжить оплату. 2 – Неудача. Прервать процесс оплаты
     * @param string $desc описание ошибки или успеха
     * @return string xml
     */
    private function xmlResult($code, $desc)
    {
        return '<result><code>' . $code . '</code><desc><![CDATA[' . $desc . ']]></desc></result>';
    }

    /**
     * Возвращает отрицательный ответ на запрос проверки платежа от платежного шлюза
     * @param string $error причина неудачи
     * @return string xml
     */
    public function checkResponseFailed($error)
    {
        if (empty($error)) {
            $error = $this->language->get('error_check_default');
        }
        return '<payment-avail-response>' . $this->xmlResult(2, $error) . '</payment-avail-response>';
    }

    /**
     * Возвращает положительный ответ на запрос проверки платежа от платежного шлюза
     * @param string $descResult Краткое описание результатов проверки. Не более 125 символов
     * @param string $descShortPurchase Краткое описание платежа. Не более 30 символов
     * @param string $descLongPurchase Полное описание платежа. Не более 125 символов. Это описание отображается клиенту на странице ввода реквизитов банковской карты
     * @param int $amount Сумма к оплате. В копейках
     * @return string xml
     * @throws Exception
     */
    public function checkResponseSuccessful($descResult, $descShortPurchase, $descLongPurchase, $amount)
    {
        if (empty($descResult)) {
            $descResult = $this->language->get('default_check_description');
        }
        if (empty($descShortPurchase)) {
            $descShortPurchase = $this->language->get('default_check_purchase_short_description');
        }
        if (empty($descLongPurchase)) {
            $descLongPurchase = $this->language->get('default_check_purchase_long_description');
        }
        $amount = (int)$amount;
        $settings = $this->settingsModel->getSettingsValue();
        // @todo: передавать информацию о покупателе доп. тегах - ФИО, телефон, email
        return '<payment-avail-response>' . $this->xmlResult(1, $descResult)
            . '<purchase>'
            . '<shortDesc><![CDATA[' . $descShortPurchase . ']]></shortDesc>'
            . '<longDesc><![CDATA[' . $descLongPurchase . ']]></longDesc>'
            . '<account-amount>'
            . '<amount>' . $amount . '</amount>'
            . '<currency>' . $settings['currency'] . '</currency>'
            . '<exponent>2</exponent>'
            . '</account-amount>'
            . '</purchase>'
            . '</payment-avail-response>';
    }

    /**
     * Возвращает положительный ответ на запрос фиксации платежа от платежного шлюза
     * @param string $desc описание успешной регистрации платежа
     * @return string xml
     */
    public function registerResponseSuccessful($desc)
    {
        if (empty($desc)) {
            $desc = $this->language->get('default_register_description');
        }
        return '<register-payment-response>' . $this->xmlResult(1, $desc) . '</register-payment-response>';
    }

    /**
     * Возвращает отрицательный ответ на запрос фиксации платежа от платежного шлюза
     * @param string $error ошибка
     * @return string xml
     */
    public function registerResponseFailed($error)
    {
        if (empty($error)) {
            $error = $this->language->get('error_register_default');
        }
        return '<register-payment-response>' . $this->xmlResult(2, $error) . '</register-payment-response>';
    }

    /**
     * Преобразуем рубли в копейки
     * @param float $amount
     * @return int
     */
    public function rublesToPennies($amount)
    {
        $amount = number_format($amount, 2, '', '');
        return (int)$amount;
    }

    /**
     * Сохранение информации об обмене с платежным шлюзом
     * @param string $trxId id платежной транзакции
     * @param int $orderId id заказа
     * @param array $info массив с данными
     */
    public function saveTransactionPayment($trxId, $orderId, $info, $ts)
    {
        if (empty($trxId) || empty($orderId) || empty($info)) {
            return ;
        }
        if (!is_array($info)) {
            return ;
        }
        $q = 'SELECT *
            FROM ' . DB_PREFIX . "delo_transaction_payment
            WHERE trx_id = '" . $this->db->escape($trxId) . "' AND order_id = " . (int)$orderId;
        $stm = $this->db->query($q);
        if (!empty($stm->row)) {
            $infoFromDb = json_decode($stm->row['info'], true);
            $infoNew = array_merge_recursive($infoFromDb, $info);
            $this->updateTransactionPayment($trxId, $orderId, $infoNew);
        } else {
            $this->insertTransactionPayment($trxId, $orderId, $info, $ts);
        }
    }

    /**
     * Вставляет информацию в таблицу платежных транзакции
     * @param string $trxId id платежной транзакции
     * @param int $orderId id заказа
     * @param array $info массив с данными
     * @param string $ts дата и время
     */
    private function insertTransactionPayment($trxId, $orderId, $info, $ts)
    {
        $jsonInfo = json_encode($info);
        $q = 'INSERT INTO ' . DB_PREFIX . 'delo_transaction_payment (
                order_id,
                trx_id,
                info,
                ts
            ) VALUE (
                ' . (int)$orderId . ",
                '" . $this->db->escape($trxId) . "',
                '" . $this->db->escape($jsonInfo) . "',
                '" . $this->formateTs($ts) . "'
            )";
        $this->db->query($q);
    }

    /**
     * Преобразует ts к формату Y-m-d H:i:s, который испольузется в БД или создает как now() ts в случае если он пустой
     * @param string $ts дата,время в формате Ymd H:i:s, например 20190829 13:57:09
     * @return string дата и время в формате Y-m-d H:i:s
     */
    private function formateTs($ts)
    {
        $ts = \DateTime::createFromFormat('Ymd H:i:s', $ts);
        if ($ts === false) {
            // Если произошла ошибка
            $ts = date('Y-m-d H:i:s');
        } else {
            $ts = $ts->format('Y-m-d H:i:s');
            if ($ts === false) {
                // На все случае now()
                $ts = date('Y-m-d H:i:s');
            }
        }
        return $ts;
    }

    /**
     * Обновляет инфомацию в таблице платежных транзакции
     * @param string $trxId id платежной транзакции
     * @param int $orderId id заказа
     * @param array $info массив с данными
     */
    private function updateTransactionPayment($trxId, $orderId, $info)
    {
        $jsonInfo = json_encode($info);
        $q = 'UPDATE ' . DB_PREFIX . "delo_transaction_payment
            SET info = '" . $this->db->escape($jsonInfo) . "'
            WHERE order_id = " . (int)$orderId . "
                AND trx_id = '" . $this->db->escape($trxId) . "'";
        $this->db->query($q);
    }

    /**
     * Устанавливает заказу статус ожидание оплаты
     * @param int $orderId
     * @param array $settings
     * @throws Exception
     */
    public function setPendingStatusOrder($orderId, $settings)
    {
        $this->load->model('checkout/order');
        $this->model_checkout_order->addOrderHistory($orderId, $settings['order_status_before']);
    }

    /**
     * Устанавливает заказу статус оплата прошла успешно
     * @param int $orderId
     * @param array $settings
     * @throws Exception
     */
    public function setCompleteStatusOrder($orderId, $settings)
    {
        $this->load->model('checkout/order');
        $this->model_checkout_order->addOrderHistory($orderId, $settings['order_status_after']);
    }

    /**
     * Устанавливает заказу статус что-то пошло не так
     * @param int $orderId
     * @param array $settings
     * @throws Exception
     */
    public function setFailedStatusOrder($orderId, $settings)
    {
        $this->load->model('checkout/order');
        $this->model_checkout_order->addOrderHistory($orderId, $settings['order_status_failed']);
    }

    /**
     * Функция для проверки цифровой подписи URI. Возвращает true в случае успеха.
     * @param string $signature ЦП из запроса
     * @param int $enableCheck Вкл./выкл. проверки ЦП
     * @return bool
     * @throws Exception
     */
    public function checkSignature($signature, $enableCheck = 1)
    {
        $debugId = $this->debugInfo->set('Проверка ЦП включена: ');
        if (empty($enableCheck)) {
            $this->debugInfo->setOk($debugId, 'НЕТ');
            // Если проверка ЦП выключена
            return true;
        }
        $this->debugInfo->setOk($debugId, 'ДА');
        $debugId = $this->debugInfo->set('Переменная окружения для определения тестовая это среда или нет ');
        if (isset($_SERVER['DELOBANK_ENVIRONMENT'])) {
            $certFile = 'file://' . DIR_SYSTEM . 'library/Delo/cert/delobank_test.crt';
            $this->debugInfo->setOk($debugId, 'определена и равна "' . $_SERVER['DELOBANK_ENVIRONMENT'] . '", используем файл "' . DIR_SYSTEM . 'library/Delo/cert/delobank_test.crt"');
        } else {
            $certFile = 'file://' . DIR_SYSTEM . 'library/Delo/cert/delobank_production.crt';
            $this->debugInfo->setOk($debugId, 'не определена, используем файл "' . DIR_SYSTEM . 'library/Delo/cert/delobank_production.crt"');
        }
        $data = $this->getDataFromUrl();
        // Преобразуем подпись в двоичный формат
        $debugId = $this->debugInfo->set('$signature = "' . $signature . '". Преобразуем подпись в двоичный формат... ');
        $signature = base64_decode($signature);
        $this->debugInfo->setOk($debugId);
        // Получаем из сертификата публичный ключ для проверки
        $this->debugInfo->set('Получаем из сертификата публичный ключ для проверки... ');
        $public_key = openssl_pkey_get_public($certFile);
        $this->debugInfo->set(print_r($public_key, true));
        $this->debugInfo->set('ok');
        $debugId = $this->debugInfo->set('Проверяем ЦП... ');
        $check = openssl_verify($data, $signature, $public_key, OPENSSL_ALGO_SHA1);
        $this->debugInfo->setOk($debugId);
        $this->debugInfo->set('Проверка вернула: ' . print_r($check, true));
        openssl_free_key($public_key);
        if ($check !== 1) {
            return false;
        }
        return true;
    }

    /**
     * Выделяет подписанные данные из запроса
     * @return string часть url-а c доменом до &signature=
     */
    private function getDataFromUrl()
    {
        $debugMessage = "Собираем часть url, которая подписана... \n";
        $url = "https://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $debugMessage .= 'Полный url: "' . $url . '" ' . "\n";
        $pos = strpos($url, '&signature=');
        $dataUrl = substr($url, 0, $pos);
        $debugMessage .= 'Получили такую часть url-а: "' . $dataUrl . '" ' . "\n";
        $this->debugInfo->set($debugMessage);
        return $dataUrl;
    }

    /**
     * @param string $trxId
     * @param int $orderId
     * @param string $response
     * @param array $requestGetData
     * @param string $ts
     */
    public function logPaymentAvail($trxId, $orderId, $response, $requestGetData, $ts)
    {
        if (empty($trxId) || empty($orderId) || empty($info)) {
            return ;
        }
        $info = [
            'PaymentAvail' => [
                [
                    'PaymentAvail_Request'  => $requestGetData,
                    'PaymentAvail_Response' => $response,
                ]
            ]
        ];
        $this->saveTransactionPayment($trxId, $orderId, $info, $ts);
    }

    /**
     * @param string $trxId
     * @param int $orderId
     * @param string $response
     * @param array $requestGetData
     * @param string $ts
     */
    public function logRegisterPayment($trxId, $orderId, $response, $requestGetData, $ts)
    {
        if (empty($trxId) || empty($orderId) || empty($info)) {
            return ;
        }
        $info = [
            'RegisterPayment' => [
                [
                    'RegisterPayment_Request' => $requestGetData,
                    'RegisterPayment_Response' => $response,
                ]
            ]
        ];
        $this->saveTransactionPayment($trxId, $orderId, $info, $ts);
    }
}
